# aircache command

from django.core.management.base import BaseCommand
from airplane.utils import cache_url, get_cache_path, convert_url

class Command(BaseCommand):
    help = 'Caches any URLs passed in as arguments in the airplane cache'

    def add_arguments(self, parser):
        parser.add_argument('url', nargs='+', type=str)

    def handle(self, *args, **options):
        dir_path = get_cache_path()

        for url in options['url']:
            filename = convert_url(url)
            cache_url(dir_path, filename, url)



import os, shutil

from django.conf import settings
from django.core.management import call_command
from django.template import Context, Template
from django.test import TestCase, override_settings
import six

import airplane
from airplane.utils import convert_url
from mock import patch
from context_temp import temp_directory
from waelstow import capture_stdout

# ============================================================================
# Test Objects
# ============================================================================

class FakeRequests(object):
    ok = True
    content = [six.b('pretend content'), ]

    def iter_content(self, **kwargs):
        return iter(self.content)


def fake_get(*args, **kwargs):
    return FakeRequests()


def fake_bad_get(*args, **kwargs):
    r = FakeRequests()
    r.ok = False
    return r


class AirplaneTests(TestCase):
    def setUp(self):
        self.cache1 = '.airplane_cache'
        self.cache1_path = os.path.join(settings.BASE_DIR, self.cache1)
        self.cache2 = '.airplane_cache2'
        self.cache2_path = os.path.join(settings.BASE_DIR, self.cache2)

    def tearDown(self):
        self._remove_local_caches()

    def assertFile(self, dir_name, filename):
        self.assertTrue(os.path.exists(os.path.join(dir_name, filename)))

    def _remove_local_caches(self):
        if os.path.exists(self.cache1_path):
            shutil.rmtree(self.cache1_path)

        if os.path.exists(self.cache2_path):
            shutil.rmtree(self.cache2_path)

    def _render(self,url, expected):
        # renders a template with our keyword in it
        t = """{% load airplanetags %}{% airplane '""" + url + """' %}"""

        template = Template(t)
        context = Context({})
        result = template.render(context)
        self.assertEqual(expected, result.strip())

    def _check_build_cache(self, url, expected_url, dir_name, 
            expected_filename):
        # test with a mocked request that works
        with patch('requests.get') as mock_requests:
            mock_requests.side_effect = fake_get

            # render the tag
            self._render(url, expected_url)

            # check that the directory got a file
            self.assertFile(dir_name, expected_filename)

    def test_airplane(self):
        url = 'http://foo.com'
        expected_filename = convert_url(url)
        expected_url = '/static/' + expected_filename

        # test pass through, DEBUG=False, no AIRPLANE_MODE set
        self._render(url, url)

        # test pass through, DEBUG=True, no AIRPLANE_MODE set
        with override_settings(DEBUG=True):
            self._render(url, url)

        # test absolute path cache creation and fetching
        with temp_directory() as td:
            with override_settings(
                    DEBUG=True,
                    AIRPLANE_CACHE=td,
                    AIRPLANE_MODE=airplane.BUILD_CACHE):

                # test with a mocked request that works
                self._check_build_cache(url, expected_url, td, 
                    expected_filename)

                # test with a mocked request that fails (404s etc)
                with patch('requests.get') as mock_requests:
                    mock_requests.side_effect = fake_bad_get

                    # render the tag
                    with self.assertRaises(IOError):
                        self._render(url, expected_url)

        # test cache creation with a local directory that needs creating
        self._remove_local_caches()

        # test with the default cache name
        with override_settings(
                DEBUG=True,
                AIRPLANE_MODE=airplane.BUILD_CACHE):

            self._check_build_cache(url, expected_url, self.cache1_path, 
                expected_filename)

        # test with a non-default cache name that is local
        with override_settings(
                DEBUG=True,
                AIRPLANE_CACHE=self.cache2,
                AIRPLANE_MODE=airplane.BUILD_CACHE):

            self._check_build_cache(url, expected_url, self.cache2_path, 
                expected_filename)

    def test_commands(self):
        # test aircache command
        with temp_directory() as td:
            with override_settings(
                    DEBUG=True,
                    AIRPLANE_CACHE=td):

                with patch('requests.get') as mock_requests:
                    mock_requests.side_effect = fake_get

                    url = 'http://foo.com/thing'
                    call_command('aircache', url)

                    filename = convert_url(url)

                    # check that the directory got a file
                    self.assertTrue(os.path.exists(os.path.join(td, filename)))

                # run airinfo and make sure it doesn't blow up
                with capture_stdout():
                    with override_settings(AIRPLANE_MODE=airplane.BUILD_CACHE):
                        call_command('airinfo')

                    with override_settings(AIRPLANE_MODE=airplane.USE_CACHE):
                        call_command('airinfo')

                    with override_settings(AIRPLANE_MODE=0):
                        call_command('airinfo')



# airplane.templatetags.airplanetags.py
from django import template
from django.conf import settings

import airplane as package
from airplane.utils import get_cache_path, convert_url, cache_url

register = template.Library()

# ============================================================================

@register.simple_tag
def airplane(url):
    """This template tag modifies a URL depending on the values in settings.
    It either returns the URL as is, returns the URL as is and caches a copy,
    or returns a re-written URL pointing to the cache.
    
    Example::

        {% airplane 'https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/css/bootstrap.min.css' %}
    """
    debug = getattr(settings, 'DEBUG', False)
    if not debug:
        # we are not in debug mode, just pass through the URL
        return url

    conf = getattr(settings, 'AIRPLANE_MODE', 0)
    if conf == 0:
        # not in AIRPLANE_MODE, pass through
        return url

    # convert url to local path
    filename = convert_url(url)
    dir_path = get_cache_path()

    if conf == package.BUILD_CACHE:
        cache_url(dir_path, filename, url)

    # we're caching, return the re-written static URL, need to encode
    return '/static/%s' % filename



import os, requests

from django.conf import settings
from django.utils.http import urlquote_plus, urlunquote_plus

import airplane

def get_cache_path():
    dirname = getattr(settings, 'AIRPLANE_CACHE', airplane.CACHE_DIR)

    if os.path.isabs(dirname):
        dir_path = dirname
    else:
        dir_path = os.path.join(getattr(settings, 'BASE_DIR'), dirname)

    return dir_path


def convert_url(url):
    c = urlquote_plus(url)
    c = c.replace('%', '|')
    return c


def reverse_convert_url(name):
    r = name
    r = r.replace('|', '%')
    r = urlunquote_plus(r)
    return r


def cache_url(dir_path, filename, url):
    # fetch the content for caching
    if not os.path.exists(dir_path):
        os.mkdir(dir_path)

    file_path = os.path.join(dir_path, filename)

    response = requests.get(url, stream=True)
    if not response.ok:
        raise IOError('Unable to fetch %s' % url)
    with open(file_path, 'wb') as stream:
        for chunk in response.iter_content(chunk_size=128):
            stream.write(chunk)



# airinfo command
from __future__ import print_function
import os

from django.conf import settings
from django.core.management.base import BaseCommand

import airplane
from airplane.utils import get_cache_path, reverse_convert_url

class Command(BaseCommand):
    help = ('Shows information about the airplane app, including the mode, ',
        'the cache directory and what is in the cache')

    def handle(self, *args, **options):
        conf = getattr(settings, 'AIRPLANE_MODE', 0)
        if conf == airplane.USE_CACHE:
            mode = 'USE_CACHE'
        elif conf == airplane.BUILD_CACHE:
            mode = 'BUILD_CACHE'
        else:
            mode = 'OFF'

        print('Cache mode:', mode)

        path = get_cache_path()
        print('Cache directory:', path)

        files = []
        for name in os.listdir(path):
            files.append(reverse_convert_url(name))

        print('Cache contents:')
        files.sort()
        for name in files:
            print('  ', name)

        print()
